/**
 *  Copyright (c) 1997-2013, tinygroup.org (luo_guo@live.cn).
 *
 *  Licensed under the GPL, Version 3.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *       http://www.gnu.org/licenses/gpl.html
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 * --------------------------------------------------------------------------
 *  版权 (c) 1997-2013, tinygroup.org (luo_guo@live.cn).
 *
 *  本开源软件遵循 GPL 3.0 协议;
 *  如果您不遵循此协议，则不被允许使用此文件。
 *  你可以从下面的地址获取完整的协议文本
 *
 *       http://www.gnu.org/licenses/gpl.html
 */
package org.tinygroup.service.registry.impl;

import java.util.Collection;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Set;

import org.tinygroup.logger.LogLevel;
import org.tinygroup.logger.Logger;
import org.tinygroup.logger.LoggerFactory;
import org.tinygroup.service.Service;
import org.tinygroup.service.registry.ServiceRegistry;
import org.tinygroup.service.registry.ServiceRegistryItem;

public class ServiceRegistryImpl implements ServiceRegistry {
	private static final String SEPARATOR = "-|-";
	/**
	 * 指定服务ID，默认服务MAP
	 */
	private final Map<String, ServiceRegistryItem> serviceIdMap = new HashMap<String, ServiceRegistryItem>();
	/**
	 * 指定服务，指定版本服务Map
	 */
	private final Map<String, ServiceRegistryItem> serviceIdVersionMap = new HashMap<String, ServiceRegistryItem>();

	private final Map<String, ServiceRegistryItem> groupIdArtifactIdNameMap = new HashMap<String, ServiceRegistryItem>();
	private final Map<String, ServiceRegistryItem> groupIdArtifactIdNameVersionMap = new HashMap<String, ServiceRegistryItem>();
	private final Map<Service, ServiceRegistryItem> serviceToServiceRegistryItem = new HashMap<Service, ServiceRegistryItem>();
	private static Logger logger = LoggerFactory.getLogger(ServiceRegistryImpl.class);
	
	public void registeService(ServiceRegistryItem serviceRegistryItem) {
		logger.logMessage(LogLevel.INFO, "添加服务[serviceId:{0},version:{1},name:{2},groupId:{3},artifactId:{4}]",
				serviceRegistryItem.getServiceId(),serviceRegistryItem.getVersion()
				,serviceRegistryItem.getName(),serviceRegistryItem.getGroupId(),serviceRegistryItem.getArtifactId());
		serviceIdMap.put(serviceRegistryItem.getServiceId(),
				serviceRegistryItem);
		serviceIdVersionMap.put(
				concat(serviceRegistryItem.getServiceId(),
						serviceRegistryItem.getVersion()), serviceRegistryItem);
		groupIdArtifactIdNameMap.put(
				concat(serviceRegistryItem.getGroupId(),
						serviceRegistryItem.getArtifactId(),
						serviceRegistryItem.getName()), serviceRegistryItem);
		groupIdArtifactIdNameVersionMap.put(
				concat(serviceRegistryItem.getGroupId(),
						serviceRegistryItem.getArtifactId(),
						serviceRegistryItem.getName(),
						serviceRegistryItem.getVersion()), serviceRegistryItem);
		serviceToServiceRegistryItem.put(serviceRegistryItem.getService(),
				serviceRegistryItem);

	}

	private String concat(Object... parameters) {
		StringBuffer sb = new StringBuffer();
		sb.append(parameters[0]);
		if (parameters.length > 1) {
			for (int index = 1; index < parameters.length; index++) {
				sb.append(SEPARATOR);
				sb.append(parameters[index]);
			}
		}
		return sb.toString();
	}

	public void registeService(List<ServiceRegistryItem> serviceRegistryItems) {
		for (ServiceRegistryItem serviceRegistryItem : serviceRegistryItems) {
			registeService(serviceRegistryItem);
		}
	}

	public void registeService(ServiceRegistryItem[] serviceRegistryItems) {
		for (ServiceRegistryItem serviceRegistryItem : serviceRegistryItems) {
			registeService(serviceRegistryItem);
		}
	}

	public void registeService(Set<ServiceRegistryItem> serviceRegistryItems) {
		for (ServiceRegistryItem serviceRegistryItem : serviceRegistryItems) {
			registeService(serviceRegistryItem);
		}
	}

	private void removeService(ServiceRegistryItem serviceRegistryItem) {
		if(serviceRegistryItem==null){
			return;
		}
		logger.logMessage(LogLevel.INFO, "删除服务[serviceId:{0},version:{1},name:{2},groupId:{3},artifactId:{4}]",
				serviceRegistryItem.getServiceId(),serviceRegistryItem.getVersion()
				,serviceRegistryItem.getName(),serviceRegistryItem.getGroupId(),serviceRegistryItem.getArtifactId());
		serviceIdMap.remove(serviceRegistryItem.getServiceId());
		serviceIdVersionMap.remove(concat(serviceRegistryItem.getServiceId(),
				serviceRegistryItem.getVersion()));
		groupIdArtifactIdNameMap.remove(concat(
				serviceRegistryItem.getGroupId(),
				serviceRegistryItem.getArtifactId(),
				serviceRegistryItem.getName()));
		groupIdArtifactIdNameVersionMap
				.remove(concat(serviceRegistryItem.getGroupId(),
						serviceRegistryItem.getArtifactId(),
						serviceRegistryItem.getName(),
						serviceRegistryItem.getVersion()));
		serviceToServiceRegistryItem.remove(serviceRegistryItem.getService());
	}

	public int size() {
		return serviceToServiceRegistryItem.size();
	}

	public void clear() {
		serviceIdMap.clear();
		serviceIdVersionMap.clear();
		groupIdArtifactIdNameMap.clear();
		groupIdArtifactIdNameVersionMap.clear();
		serviceToServiceRegistryItem.clear();
	}

	public ServiceRegistryItem getServiceRegistryItem(Service service) {
		return serviceToServiceRegistryItem.get(service);
	}

	public ServiceRegistryItem getServiceRegistryItem(String serviceId) {
		return serviceIdMap.get(serviceId);
	}

	public ServiceRegistryItem getServiceRegistryItem(String serviceId,
			String version) {
		if(version==null||"".endsWith(version)){
			return getServiceRegistryItem(serviceId);
		}
		return serviceIdVersionMap.get(concat(serviceId, version));
	}

	public ServiceRegistryItem getServiceRegistryItem(String groupId,
			String artifactId, String name) {
		return groupIdArtifactIdNameMap.get(concat(groupId, artifactId, name));
	}

	public ServiceRegistryItem getServiceRegistryItem(String groupId,
			String artifactId, String name, String version) {
		return groupIdArtifactIdNameVersionMap.get(concat(groupId, artifactId,
				name, version));
	}

	public void removeService(String serviceId, String version) {
		removeService(serviceIdVersionMap.get(concat(serviceId, version)));

	}

	public void removeService(String groupId, String artifactId, String name,
			String version) {
		removeService(groupIdArtifactIdNameVersionMap.get(concat(groupId,
				artifactId, name, version)));
	}

	public Collection<ServiceRegistryItem> getServiceRegistryItems() {
		return serviceIdVersionMap.values();
	}

}
